/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

/*
 * Point3D.cpp
 *
 *  Created on: Jun 4, 2010
 *      Author: lziegler
 */

#include "Point3D.h"
#include <cmath>

using namespace xmltio;
using namespace std;

namespace airobots {

const float TOLERANCE = 0.0001;
const LengthUnit iLU = MILLIMETER;

Point3D::Point3D() :
		scope(GLOBAL) {
}

Point3D::Point3D(float x, float y, float z, LengthUnit unit) :
		x(x), y(y), z(z), scope(GLOBAL) {
	setX(x, unit);
	setY(y, unit);
	setZ(z, unit);
}
Point3D::Point3D(float x, float y, float z, LengthUnit unit,
		Point3D::Scope scope) :
		x(x), y(y), z(z), scope(scope) {
	setX(x, unit);
	setY(y, unit);
	setZ(z, unit);
}

Point3D::~Point3D() {
}

std::string Point3D::getBaseTag() {
	return "POINT3D";
}

std::string Point3D::getTemplate() {
	return string("<") + getBaseTag()
			+ " x=\"\" y=\"\" z=\"\" scope=\"\"><GENERATOR/>"
			+ Timestamp::getTemplate() + "</" + getBaseTag() + ">";
}

Point3D::Scope Point3D::getScope() const {
	return scope;
}

float Point3D::getX(LengthUnit unit) const {
	return UnitConverter::convert(x, iLU, unit);
}

float Point3D::getY(LengthUnit unit) const {
	return UnitConverter::convert(y, iLU, unit);
}

float Point3D::getZ(LengthUnit unit) const {
	return UnitConverter::convert(z, iLU, unit);
}

void Point3D::setScope(Scope scope) {
	this->scope = scope;
}

void Point3D::setX(float x, LengthUnit unit) {
	this->x = UnitConverter::convert(x, unit, iLU);
}

void Point3D::setY(float y, LengthUnit unit) {
	this->y = UnitConverter::convert(y, unit, iLU);
}

void Point3D::setZ(float z, LengthUnit unit) {
	this->z = UnitConverter::convert(z, unit, iLU);
}

void Point3D::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);
	loc[XPath("@x")] = x;
	loc[XPath("@y")] = y;
	loc[XPath("@z")] = z;

	if (scope == LOCAL) {
		loc[XPath("@scope")] = "LOCAL";
	} else {
		loc[XPath("@scope")] = "GLOBAL";
	}
}

void Point3D::fromLocation(const xmltio::Location &loc_, Point3D &object) {

	Type::fromLocation(loc_, object);
	object.x = extract<float>(loc_[XPath("@x")]);
	object.y = extract<float>(loc_[XPath("@y")]);
	object.z = extract<float>(loc_[XPath("@z")]);

	string scopeStr = extract<string>(loc_[XPath("@scope")]);
	if (scopeStr == "LOCAL") {
		object.setScope(LOCAL);
	} else {
		object.setScope(GLOBAL);
	}
}

void Point3D::sanitizeLocation(xmltio::Location & loc) const {
	xmltio::XPath xp(getBaseTag());
	while (xmltio::XPathIterator i = xp.evaluate(loc)) {
		xmltio::Location l(*i);
		loc.remove(l);
	}
}

bool Point3D::operator==(const Point3D &other) const {

	if (!Type::operator ==(other))
		return false;
	if (abs(other.x - x) > TOLERANCE)
		return false;
	if (abs(other.y - y) > TOLERANCE)
		return false;
	if (abs(other.z - z) > TOLERANCE)
		return false;
	if (other.scope != scope)
		return false;

	return true;

}

ostream& operator<<(ostream& o, const Point3D& p) {

	return o << "Point3D [gen=" << p.getGenerator() << ", timestamp="
			<< p.getTimestamp() << ", coordinates=" << p.getX(iLU) << ";"
			<< p.getY(iLU) << ";" << p.getZ(iLU) << ", scope=" << p.getScope() << "]";

}
}
