/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.ListIterator;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

import org.apache.log4j.Logger;


/**
 * Generic type for lists of BTL types. Note that since the actual type of the
 * list elements is not known in a static context, calling the static methods
 * getBaseTag and fromElement could yield unexpected results. There is an
 * alternative getBaseTag(Class<T>) method that can be called with the element
 * type class and returns the base tag of the element type with the suffix LIST
 * appended to it.
 * 
 * @param <T>
 *            The type of the list elements.
 */
public class List<T extends Type> extends Type implements java.util.List<T> {

	private static final String BASE_TAG = "LIST";

	private Class<T> elementType;
	private java.util.List<T> elements;

	private static Logger logger = Logger.getLogger(Type.class);
	
	@SuppressWarnings("unchecked")
	public static <S extends List<T>, T extends Type> Class<S> getListClass(Class<T> type) {
		return (Class<S>) new List<T>(type).getClass();
	}

	/**
	 * Create a new list.
	 * 
	 * @param type
	 *            The type of the elements.
	 */
	public List(Class<T> type) {
		elements = new ArrayList<T>();
		elementType = type;
	}

	/**
	 * Getter for the type of the elements.
	 * 
	 * @return The type of the elements.
	 */
	public Class<T> getElementType() {
		return elementType;
	}

	@Override
	public boolean add(T t) {
		return elements.add(t);
	}

	@Override
	public T get(int index) {
		return elements.get(index);
	}

	@Override
	public boolean contains(Object o) {
		return elements.contains(o);
	}

	@Override
	public int size() {
		return elements.size();
	}

	/**
	 * Returns the tag name of the the root tag this type is usually used with
	 * in xml.
	 * 
	 * @return tag name
	 */
	public static String getBaseTag() {
		return BASE_TAG;
	}

	/**
	 * Returns the tag name of the the element tags this type is usually used
	 * with in xml.
	 * 
	 * @return tag name
	 * @param <T>
	 *            type of the list's elements
	 * @param type
	 *            Class object of the list element's type
	 */
	public static <T extends Type> String getBaseTag(Class<T> type) {
		String elementBaseTag = "";

		try {
			elementBaseTag = (String) type.getMethod("getBaseTag").invoke(null);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}

		return elementBaseTag + "LIST";
	}

	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		for (T p : elements) {

			try {

				String baseTag = (String) elementType.getMethod("getBaseTag")
						.invoke(null);
				Element perceptElement = new Element(baseTag);
				p.fillInto(perceptElement);
				parent.appendChild(perceptElement);

			} catch (IllegalArgumentException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (SecurityException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (IllegalAccessException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (InvocationTargetException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (NoSuchMethodException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			}

		}
	}

	/**
	 * Parses the contents of the given nu.xom.Element into the type instance.
	 * Therefore element must already be parsed into the base tag of the list
	 * type.
	 * 
	 * @param element
	 *            element to get the contents of this type from
	 * @param type
	 *            type to fill with the contents
	 * @param <T>
	 *            type of the list's elements.
	 */
	public static <T extends Type> void fromElement(Element element,
			List<T> type) {

		Type.fromElement(element, type);

		try {

			String baseTag = (String) type.getElementType()
					.getMethod("getBaseTag").invoke(null);
			Nodes nodes = element.query(baseTag);

			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);

				if (node instanceof Element) {
					Element perceptElement = (Element) node;

					T parsed = type.getElementType().newInstance();
					Method fromElement = type.getElementType()
							.getMethod(
									"fromElement",
									new Class[] { Element.class,
											type.getElementType() });
					fromElement.invoke(null, perceptElement, parsed);

					type.add(parsed);
				}
			}

		} catch (InstantiationException e) {
			logger.fatal("Reflection error. Types must have a default "
					+ "constructor.");
			logger.debug(e);
			assert false : "Types must have a default constructor.";
		} catch (IllegalAccessException e) {
			logger.fatal("Reflection error. Types must have a default "
					+ "constructor.");
			logger.debug(e);
			assert false : "Types must have a default constructor.";
		} catch (SecurityException e) {
			logger.fatal("Reflection error. SecurityException");
			logger.debug(e);
			assert false : "Reflection error. SecurityException";
		} catch (NoSuchMethodException e) {
			logger.fatal("Reflection error. NoSuchMethodException");
			logger.debug(e);
			assert false : "Reflection error.";
		} catch (IllegalArgumentException e) {
			logger.fatal("Reflection error. IllegalArgumentException");
			logger.debug(e);
			assert false : "Reflection error. IllegalArgumentException";
		} catch (InvocationTargetException e) {
			logger.fatal("Reflection error. InvocationTargetException");
			logger.debug(e);
			assert false : "Reflection error. InvocationTargetException";
		}
	}

	@Override
	public void add(int index, T element) {
		elements.add(index, element);
	}

	@Override
	public boolean addAll(Collection<? extends T> c) {
		return elements.addAll(c);
	}

	@Override
	public boolean addAll(int index, Collection<? extends T> c) {
		return elements.addAll(index, c);
	}

	@Override
	public void clear() {
		elements.clear();
	}

	@Override
	public boolean containsAll(Collection<?> c) {
		return elements.containsAll(c);
	}

	@Override
	public int indexOf(Object o) {
		return indexOf(o);
	}

	@Override
	public boolean isEmpty() {
		return elements.isEmpty();
	}

	@Override
	public Iterator<T> iterator() {
		return elements.iterator();
	}

	@Override
	public int lastIndexOf(Object o) {
		return elements.lastIndexOf(o);
	}

	@Override
	public ListIterator<T> listIterator() {
		return elements.listIterator();
	}

	@Override
	public ListIterator<T> listIterator(int index) {
		return elements.listIterator(index);
	}

	@Override
	public boolean remove(Object o) {
		return elements.remove(o);
	}

	@Override
	public T remove(int index) {
		return elements.remove(index);
	}

	@Override
	public boolean removeAll(Collection<?> c) {
		return elements.removeAll(c);
	}

	@Override
	public boolean retainAll(Collection<?> c) {
		return elements.retainAll(c);
	}

	@Override
	public T set(int index, T element) {
		return elements.set(index, element);
	}

	@Override
	public java.util.List<T> subList(int fromIndex, int toIndex) {
		return elements.subList(fromIndex, toIndex);
	}

	@Override
	public Object[] toArray() {
		return elements.toArray();
	}

	@Override
	public <T2> T2[] toArray(T2[] a) {
		return elements.toArray(a);
	}
}
