/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.person;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.units.*;

/**
 * The legs class stores information about leg hypothesis coming from the leg
 * detector.
 * 
 * @author: dschulze <dschulze@techfak.uni-bielefeld.de>
 */
public class Legs extends Type {

	private double angle;
	private double distance;
	private double angleVariance;
	private double distanceVariance;
	private int numInvolved;
	private int[] involved;
	private boolean pair;
	private AngleUnit iAU = AngleUnit.RADIAN;
	private LengthUnit iLU = LengthUnit.MILLIMETER;

	public Legs() {

		involved = new int[2];
		involved[0] = -1;
		involved[1] = -1;

	}

	public void setAngle(double angle,AngleUnit aU) {
		this.angle = UnitConverter.convert(angle, aU, iAU);
	}

	public double getAngle(AngleUnit aU) {
		return UnitConverter.convert(angle, iAU, aU);
	}

	public void setAngleVariance(double angleVariance,AngleUnit aU) {
		this.angleVariance = UnitConverter.convert(angleVariance, aU, iAU);
	}

	public double getAngleVariance(AngleUnit aU) {
		return UnitConverter.convert(angleVariance, iAU, aU);
	}

	public void setDistance(double distance,LengthUnit lU) {
		this.distance = UnitConverter.convert(distance, lU, iLU);
	}

	public double getDistance(LengthUnit lU) {
		return UnitConverter.convert(distance, iLU, lU);
	}

	public void setDistanceVariance(double distanceVariance,LengthUnit lU) {
		this.distanceVariance = UnitConverter.convert(distanceVariance, lU, iLU);
	}

	public double getDistanceVariance(LengthUnit lU) {
		return UnitConverter.convert(distanceVariance, iLU, lU);
	}

	public void setNumInvolved(int numInvolved) {
		this.numInvolved = numInvolved;
	}

	public int getNumInvolved() {
		return numInvolved;
	}

	public void setInvolved1(int involved) {
		this.involved[0] = involved;
	}

	public int getInvolved1() {
		return involved[0];
	}

	public void setInvolved2(int involved) {
		this.involved[1] = involved;
	}

	public int getInvolved2() {
		return involved[1];
	}

	public void setPair(boolean pair) {
		this.pair = pair;
	}

	public boolean isPair() {
		return pair;
	}

	public static String getBaseTag() {
		return "LEGS";
	}

	/** @{inheritDoc */
	public static void fromElement(Element element, Legs legs) {

		Type.fromElement(element, legs);

		Nodes nodes = element.query("*");
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element elem = (Element) node;
				if (elem.getLocalName().equals("POSITION")) {

					double tmp = ElementParser.getDoubleAttributeValue(elem,
							"angle");
					legs.setAngle(tmp,AngleUnit.RADIAN);

					tmp = ElementParser.getDoubleAttributeValue(elem,
							"angleVariance");
					legs.setAngleVariance(tmp,AngleUnit.RADIAN);

					tmp = ElementParser.getDoubleAttributeValue(elem,
							"distance");
					legs.setDistance(tmp,LengthUnit.MILLIMETER);

					tmp = ElementParser.getDoubleAttributeValue(elem,
							"distanceVariance");
					legs.setDistanceVariance(tmp,LengthUnit.MILLIMETER);

				}

				if (elem.getLocalName().equals("SHARING")) {

					int tmp = ElementParser.getIntAttributeValue(elem,
							"nrInvolved");
					legs.setNumInvolved(tmp);

					tmp = ElementParser.getIntAttributeValue(elem, "involved1");
					legs.setInvolved1(tmp);

					tmp = ElementParser.getIntAttributeValue(elem, "involved2");
					legs.setInvolved2(tmp);

					boolean pair = ElementParser.getBooleanAttributeValue(elem,
							"isPair");
					legs.setPair(pair);

				}

			}

		}

	}

	/** @{inheritDoc */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element positionElement = new Element("POSITION");
		positionElement.addAttribute(new Attribute("angle", String
				.valueOf(getAngle(AngleUnit.RADIAN))));
		positionElement.addAttribute(new Attribute("angleVariance", String
				.valueOf(getAngleVariance(AngleUnit.RADIAN))));
		positionElement.addAttribute(new Attribute("distance", String
				.valueOf(getDistance(LengthUnit.MILLIMETER))));
		positionElement.addAttribute(new Attribute("distanceVariance", String
				.valueOf(getDistanceVariance(LengthUnit.MILLIMETER))));

		parent.appendChild(positionElement);

		Element sharingElement = new Element("SHARING");
		sharingElement.addAttribute(new Attribute("nrInvolved", String
				.valueOf(getNumInvolved())));
		sharingElement.addAttribute(new Attribute("involved1", String
				.valueOf(getInvolved1())));
		sharingElement.addAttribute(new Attribute("involved2", String
				.valueOf(getInvolved2())));
		sharingElement.addAttribute(new Attribute("isPair", String
				.valueOf(isPair())));

		int pair = (isPair()) ? 1 : 0;
		sharingElement.addAttribute(new Attribute("isPair", String.valueOf(pair)));

		parent.appendChild(sharingElement);

	}

}
