/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.status;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import de.unibi.airobots.btl.Type;

import nu.xom.Element;
import nu.xom.Elements;

/**
 * Xml type for the state visualizer configuration.
 * 
 * @author jwienke
 */
public class StateConfiguration extends Type {

	private static final String STATE_ELEMENT = "STATE";
	private static final String SYSTEM_STATES_ELEMENT = "SYSTEMSTATES";
	private static final String SYSTEM_NAME_ELEMENT = "SYSTEMNAME";
	private String systemName = "unknown";
	private Set<String> states = new HashSet<String>();

	/**
	 * Constructor.
	 */
	public StateConfiguration() {
	}

	/**
	 * Adds a new state.
	 * 
	 * @param stateName
	 *            name of the new state.
	 */
	public void addState(String stateName) {
		this.states.add(stateName);
	}

	/**
	 * Adds all specified states.
	 * 
	 * @param stateNames
	 *            collection of states to add
	 */
	public void addStates(Collection<String> stateNames) {
		this.states.addAll(stateNames);
	}

	/**
	 * Returns all states.
	 * 
	 * @return states
	 */
	public Set<String> getStates() {
		return states;
	}

	/**
	 * Removes all known states.
	 */
	public void removeAllStates() {
		states.clear();
	}

	/**
	 * Returns the name of the system to configure.
	 */
	public String getSystemName() {
		return systemName;
	}

	/**
	 * Sets the name of the system.
	 * 
	 * @param systemName
	 *            new name
	 */
	public void setSystemName(String systemName) {
		this.systemName = systemName;
	}

	public static String getBaseTag() {
		return "STATEVIZCONFIG";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element nameElement = new Element(SYSTEM_NAME_ELEMENT);
		parent.appendChild(nameElement);
		nameElement.appendChild(getSystemName());

		Element statesElement = new Element(SYSTEM_STATES_ELEMENT);
		parent.appendChild(statesElement);
		for (String s : getStates()) {
			Element currentStateElement = new Element(STATE_ELEMENT);
			statesElement.appendChild(currentStateElement);
			currentStateElement.appendChild(s);
		}

	}

	public static void fromElement(Element element, StateConfiguration type) {

		Type.fromElement(element, type);

		type.setSystemName(element.getFirstChildElement(SYSTEM_NAME_ELEMENT)
				.getValue());

		Element statesElement = element
				.getFirstChildElement(SYSTEM_STATES_ELEMENT);
		Elements stateElements = statesElement.getChildElements(STATE_ELEMENT);
		for (int i = 0; i < stateElements.size(); i++) {
			type.addState(stateElements.get(i).getValue());
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "[StateConfiguration: systeName = " + getSystemName()
				+ ", states = " + states.toString() + "]";
	}

}
