/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.vision1d;

import java.util.StringTokenizer;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.common.MicroTimestamp;
import de.unibi.airobots.btl.data.common.Ticked;
import de.unibi.airobots.btl.data.hardwareinfo.LaserInfo;
import de.unibi.airobots.btl.data.hardwareinfo.LaserInfo.OutOfRangeException;
import de.unibi.airobots.btl.data.hardwareinfo.LaserInfo.ScannerModel;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;
import de.unibi.airobots.btl.units.UnitConverter;

/**
 * Domain class for LaserData of BonSAI. The currently mounted Laser has a
 * resolution of 0.5 degrees and scans an angle of 180 degrees in front of the
 * robot, resulting in 361 values. Use LaserData.getAngleValue to obtain the
 * corresponding angles of value. The laser range is limited to 8 meters. The
 * resolution is about 10mm.
 * 
 * @author marc
 * @author jwienke
 * @author lziegler
 */
public class LaserData extends Type implements Ticked {

	private final static Logger logger = Logger.getLogger(LaserData.class);

	private double[] scanValues;
	private MicroTimestamp readTime = new MicroTimestamp();
	private static AngleUnit iAU = AngleUnit.RADIAN;
	private static LengthUnit iLU = LengthUnit.METER;

	/**
	 * @return the of values in the scanValues Array. Normally 361.
	 */
	public int getNumLaserPoints() {
		return scanValues.length;
	}

	/**
	 * get the value of the angle corresponding to the value at index.
	 * 
	 * @param index
	 *            of the distance value you are interested in
	 * @return the value in radian. a value of 0 corresponds to the distance
	 *         directly in front of the robot, angles increase
	 *         counter-clock-wise. angles are always positive. So, a value of
	 *         Math.PI/2 corresponds to the left of the robot, and Math.PI*(3/2)
	 *         to its right.
	 */
	public static double getAngleValue(int index, AngleUnit u) {
		double correspondingAngleDeg = ((double) index) * Math.PI / 360.0
				- Math.PI / 2;
		if (correspondingAngleDeg < 0)
			correspondingAngleDeg += 2 * Math.PI;
		return UnitConverter
				.convert(correspondingAngleDeg, AngleUnit.DEGREE, u);
	}

	/**
	 * @return an array which hold all scan Values (normally 361). All values
	 *         are in meters. If you need to know the angle of a scan use the
	 *         getAngleValue(int index) method.
	 */
	public double[] getScanValues(LengthUnit u) {
		if (u == iLU) {
			return scanValues;
		} else {
			double[] newValues = new double[scanValues.length];
			for (int i = 0; i < scanValues.length; i++) {
				newValues[i] = UnitConverter.convert(scanValues[i], iLU, u);
			}
			return newValues;
		}
	}

	public void setScanValues(double[] scanValues, LengthUnit u) {
		if (u == iLU) {
			this.scanValues = scanValues;
		} else {
			double[] newValues = new double[scanValues.length];
			for (int i = 0; i < scanValues.length; i++) {
				newValues[i] = UnitConverter.convert(scanValues[i], u, iLU);
			}
			this.scanValues = newValues;
		}
	}

	/**
	 * Getter for the scan value at a given angle.
	 * 
	 * @param scannerModel
	 *            Identity of the used laser scanner.
	 * @param angle
	 *            The angle of the requested value. [radiant]
	 * @return The scan value at a given angle
	 * @throws OutOfRangeException
	 *             Is thrown if the given angle is out of the laser scanners
	 *             range.
	 */
	public double getSingleScanValue(double angle, AngleUnit au,
			ScannerModel scannerModel, LengthUnit lu)
			throws OutOfRangeException {

		int scanNum = LaserInfo.calcScanIndex(scannerModel,
				UnitConverter.convert(angle, au, AngleUnit.RADIAN));

		return getScanValues(lu)[scanNum];
	}

	/**
	 * Getter for an average scanner value over a given angle range.
	 * 
	 * @param scannerModel
	 *            Identity of the used laser scanner.
	 * @param angle
	 *            The angle of the requested value. [radiant]
	 * @param widthAngle
	 *            The width of the scan window used for average the value.
	 *            [radiant]
	 * @return
	 * @throws OutOfRangeException
	 *             Is thrown if the given angle is out of the laser scanners
	 *             range.
	 */
	public double getAverageScanValue(double angle, double widthAngle,
			AngleUnit au, ScannerModel scannerModel, LengthUnit lu)
			throws OutOfRangeException {
		
		angle = UnitConverter.convert(angle, au, AngleUnit.RADIAN);
		widthAngle = UnitConverter.convert(widthAngle, au, AngleUnit.RADIAN);

		// calculate average window
		double startAngle = angle - widthAngle / 2.0;
		double endAngle = angle + widthAngle / 2.0;
		double rangeRad = LaserInfo.getInfo(scannerModel).getRange(
				AngleUnit.RADIAN);

		// make window smaller if it exceeds the laser's range.
		if (startAngle < -rangeRad / 2.0) {
			startAngle = -rangeRad / 2.0;
		}
		if (endAngle > rangeRad / 2.0) {
			endAngle = rangeRad / 2.0;
		}

		int scanNumStart = LaserInfo.calcScanIndex(scannerModel, startAngle);
		int scanNumEnd = LaserInfo.calcScanIndex(scannerModel, endAngle);

		// calculate average
		double value = 0;
		int count = 0;
		double maxDist = LaserInfo.getInfo(scannerModel).getMaxDistance();
		maxDist = UnitConverter.convert(maxDist, iLU, lu);
		double[] sv = getScanValues(lu);
		for (int i = scanNumStart; i <= scanNumEnd; i++) {
			if (sv[i] < maxDist) {
				value += sv[i];
				count++;
			}
		}
		value = value / (double) count;

		return value;
	}

	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("#LASER# ");
		strb.append("timestamp: " + getTimestamp() + "; ");
		strb.append("num: " + getNumLaserPoints() + "; ");
		strb.append("values:");
		for (int i = 0; i < scanValues.length; i++) {
			strb.append(" " + scanValues[i]);
		}
		strb.append(";");
		return strb.toString();
	}

	public static String getBaseTag() {
		return "LASERDATA";
	}

	public static void fromElement(Element element, LaserData type)
			throws ParsingException {

		Type.fromElement(element, type);

		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals("MEASUREMENTS")) {
					int num = ElementParser.getIntAttributeValue(elem, "num");
					StringTokenizer valuesString = new StringTokenizer(
							elem.getValue());
					int valuesInt[] = new int[valuesString.countTokens()];
					if (num != valuesString.countTokens()) {
						logger.error("value in XML differs from object count: "
								+ num + " : " + valuesString.countTokens());
					}
					int j = 0;
					while (valuesString.hasMoreTokens()) {
						valuesInt[j] = Integer.parseInt(valuesString
								.nextToken());
						j++;
					}
					double valuesDouble[] = new double[valuesInt.length];
					for (int k = 0; k < valuesInt.length; k++) {
						valuesDouble[k] = ((double) valuesInt[k] / 1000);
					}
					type.setScanValues(valuesDouble, iLU);
				} else if (elem.getLocalName().equals(
						MicroTimestamp.getBaseTag())) {
					MicroTimestamp readTime = new MicroTimestamp();
					MicroTimestamp.fromElement(elem, readTime);
					type.setReadTime(readTime);
				}
			}
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element readTimeElement = new Element(MicroTimestamp.getBaseTag());
		parent.appendChild(readTimeElement);
		getReadTime().fillInto(readTimeElement);

		Element measurementsElement = new Element("MEASUREMENTS");
		parent.appendChild(measurementsElement);
		measurementsElement.addAttribute(new Attribute("num", String
				.valueOf(getNumLaserPoints())));

		StringBuilder text = new StringBuilder();
		for (double m : getScanValues(iLU)) {
			text.append((int) Math.round(m * 1000.0));
			text.append(' ');
		}

		measurementsElement.appendChild(text.toString());

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public MicroTimestamp getReadTime() {
		return readTime;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeMicroSeconds() {
		return readTime.getMicroSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeSeconds() {
		return readTime.getSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setReadTime(MicroTimestamp readTime) {
		this.readTime = readTime;
	}

}
